@@ -29,7 +29,8 @@ typedef unsigned long	u_long;
 
 #ifdef TCL_WRAPPER
   #define malloc Tcl_Alloc
-  #define calloc(a,b) Tcl_Alloc(a * b)
+  /* (see definition of my_calloc in the asg subdirectory) */
+  // #define calloc(a,b) Tcl_Alloc(a * b)
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
   #undef strdup
@@ -100,7 +101,9 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   ((*(topobject->plist + *(a)))->type & ALL_TYPES)
+#define SELECTTYPE(a)   ((areastruct.drawstack != NULL) ? \
+			OBJINST : \
+			((*(topobject->plist + *(a)))->type & ALL_TYPES))
 #define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
 #define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
 #define ENDPART		topobject->plist + topobject->parts
@@ -131,11 +134,15 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 
 #define SELTOPOLY(a)	TOPOLY(topobject->plist + (*(a)))
 #define SELTOLABEL(a)	TOLABEL(topobject->plist + (*(a)))
-#define SELTOOBJINST(a)	TOOBJINST(topobject->plist + (*(a)))
+#define SELTOOBJINST(a)	((areastruct.drawstack != NULL) ? \
+			areastruct.drawstack->thisinst : \
+			TOOBJINST(topobject->plist + (*(a))))
 #define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
 #define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
 #define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
-#define SELTOGENERIC(a) (*(topobject->plist + (*(a))))
+#define SELTOGENERIC(a) ((areastruct.drawstack != NULL) ? \
+			(genericptr)areastruct.drawstack->thisinst : \
+			(*(topobject->plist + (*(a)))))
 
 /* creation of new elements */
 
@@ -170,22 +177,6 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
 
-/*----------------------------------------------------------------------*/
-/* Undo mechanism definitions						*/
-/*----------------------------------------------------------------------*/
-
-enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE};
-
-/*----------------------------------------------------------------------*/
-/* selection from top-level object 					*/
-/*----------------------------------------------------------------------*/
-
-#define select_element(a) genselectelement(a, UNDO_MORE, topobject,\
-	areastruct.topinstance)
-#define select_add_element(a) genselectelement(a, UNDO_DONE, topobject,\
-	areastruct.topinstance)
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
-
 /*----------------------------------------------------------------------*/
 
 typedef struct {
@@ -681,6 +672,31 @@ typedef struct {
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
 } path, *pathptr;
 
+/*----------------------------------------------------------------------*/
+/* selection from top-level object 					*/
+/*----------------------------------------------------------------------*/
+
+/*----------------------------------------------------------------------*/
+/* Undo mechanism definitions						*/
+/*----------------------------------------------------------------------*/
+
+enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE, MODE_CONNECT, MODE_RECURSE_WIDE,
+	MODE_RECURSE_NARROW};
+
+typedef struct _selection *selectionptr;
+
+typedef struct _selection {
+   int selects;
+   short *selectlist;
+   objinstptr thisinst;
+   selectionptr next;
+} selection;
+
+#define select_element(a) recurse_select_element(a, UNDO_MORE) 
+#define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
+
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
@@ -1166,6 +1182,7 @@ typedef struct {
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
+   pushlistptr	drawstack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
